	* 68000 master controller code for
	* Theo Markettos' Home Automation System
	* 
	* This code is copyright Theo Markettos 1997,
	* and may be freely distributed, with the proviso
	* that it is used for educational and non-profit
	* making purposes only
	* 
	* If you have any problems using this code, you
	* should contact the author at
	* theomarkettos@letterbox.com
	
	INCLUDE x68.defs

trap_code	EQU 	$200
interrupt_code	EQU	$400

	INCLUDE x68.excs
	INCLUDE x68.intserv4

	ORG	$800			* base of ROM after exception table
start
	BRA	main_init		* main initialisation
start_init
	BSR	serial_init		* initialise serial system
	BSR	pia_init		* initialise PIA
	BSR	rtc_init                * initialise clock
	BSR	via_init		* set VIA to talk to PIC
	BSR	program_init




main_menu
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	LEA	mmstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for main menu, and send

	MOVEQ	#'7',D1			* maximum number to accept
	BSR	num_wait		* wait for a character
	MOVE.L	D0,D2
	MOVE.W	#$8000,D1		* provide a small delay so that the
main_menu_char_delay			* character can be seen on terminal
	DBF	D1,main_menu_char_delay
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear screen
	SUB.B	#'1',D2			* convert ASCII number into
	CMP.B	#0,D2			* binary
	BEQ	system_config
	CMP.B	#1,D2
	BEQ	control_now
	CMP.B	#2,D2
	BEQ	set_event
	CMP.B	#5,D2
	BEQ	rtc_showtime
	CMP.B	#6,D2
	BEQ	quit
	BRA	main_menu

quit
	LEA	modem_cmdmode,A0		* pointer to modem wakeup string
	BSR	send_string
	MOVEQ	#5,D1
quit_loop1
	MOVE.L	#$FFFF,D0		* Effectively a delay loop value
quit_loop2				* of $5FFFF
	DBF	D0,quit_loop2		* wait for a seconds until modem
	DBF	D1,quit_loop1		* responds
	LEA.l	modem_hangup,A0
	BSR	send_string		* send hangup command
quit_waitfornodcd
	MOVE.B	ACIA_SR,D0		* get status register
	BTST	#2,D0			* is DCD bit high
	BEQ	quit_waitfornodcd	* if not wait until it is
	CLR	D0
quit_endloop
	ADDQ	#1,D0
	*MOVE.W	D0,DOUT
	BRA	quit_endloop

system_config
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	LEA	configstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for config menu, and send
	MOVE.B	#'2',D1			* maximum number to accept
	BSR	num_wait		* get key character
	MOVE.L	D0,D2
	MOVE.W	#$8000,D1		* provide a small delay so that the
system_config_char_delay		* character can be seen on terminal
	DBF	D1,system_config_char_delay
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear screen
	SUB.B	#'1',D2			* convert ASCII number into binary
	CMP.B	#0,D2			* if 1 pressed, set time
	BEQ	rtc_settime
	CMP.B	#1,D2			* if 2 pressed, go to main menu
	BEQ	main_menu
	BRA	system_config


num_wait				* wait for a number to be entered
					* must be 1 digit, between 1 and D1
	BSR	serial_getc		* get character
	BEQ	num_wait		* if no char ready, wait for one
	CMP.B	#'1',D0			* is it a number 1-2
	BLT	num_wait		* if not wait for a number
	CMP.B	D1,D0
	BGT	num_wait
	BSR	serial_putc		* send the character
	RTS

rtc_settime
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
        BSR	send_stringtable	* print banner
	LEA	clksetstrtbl,A1
        BSR	send_stringtable	* print title

	LEA	rtcsettbl,A3		* table pointing to prompt and regs
rtc_settime_loop
	MOVEQ	#CR,D0
	BSR	serial_putc
	MOVEQ	#LF,D0
	BSR	serial_putc
	MOVEA.L	(A3)+,A2		* get prompt ptr
	MOVE.W	(A3)+,D3		* get min value
	MOVE.W	(A3)+,D4		* get max value
	BSR	get_bcd_number
	MOVE.W	(A3)+,D1		* RTC register
	MOVE.L	D0,D2			* value to store
	MOVE.W	#Hardware_RTC_Write,D0	* call OS routine
	TRAP	#15
	MOVE.L	(A3),D0			* get next address
	TST	D0			* is it zero?
	BNE	rtc_settime_loop	* if not, get next value
	BRA	system_config

get_bcd_number
	MOVEA.L	A2,A0
	BSR	send_string		* print prompt
	BSR	get_line		* read line
	BSR.S	bcdval_line		* extract BCD data from line
	*MOVE.W	D0,DOUT
	*MOVE.B	D3,DOUT_L
	CMP.W	D3,D0
	BLT.S	get_bcd_number_again
	CMP.W	D4,D0			* is it past max value
	BGT.S	get_bcd_number_again
	RTS

get_bcd_number_again
	*MOVEQ	#11,D0
	*BSR	serial_putc
	LEA	cursorup,A0
	BSR	send_string
	LEA	wipestr,A0
	BSR	send_string
	BRA	get_bcd_number

bcdval_line				* convert string pointed to by A0
	MOVEM.L	D1/A0,-(A7)		* to BCD number
	MOVE.B	(A0)+,D1		* get character
	CMP.B	#'0',D1                 * check it is a valid digit
	BLT.S	bcdval_line_err
	CMP.B	#'9',D1
	BGT.S	bcdval_line_err
	SUB.B	#'0',D1			* convert to binary digit
	LSL.W	#4,D1			* make into MSB
	MOVE.W	D1,D0			* put in D0
	MOVE.B	(A0)+,D1		* get character
	CMP.B	#32,D1
	BLT.S	bcdval_line_singdig
	*MOVE.B	#1,DOUT_H
	CMP.B	#'0',D1                 * check it is a valid digit
	BLT.S	bcdval_line_err
	*MOVE.B	#2,DOUT_H
	CMP.B	#'9',D1
	BGT.S	bcdval_line_err
	*MOVE.B	#3,DOUT_H
	SUB.B	#'0',D1			* convert to binary digit
	OR.W	D1,D0			* put in LSB of D0
	AND.W	#$FF,D0
	MOVEM.L	(A7)+,D1/A0
	RTS				* return value in D0

bcdval_line_err
	MOVEQ	#-1,D0			* return invalid result
	MOVEM.L	(A7)+,D1/A0
	RTS

bcdval_line_singdig			* if second digit is ctrl char,
	LSR.W	#4,D0			* treat as 1 digit
	AND.W	#$0F,D0
	MOVEM.L	(A7)+,D1/A0
	RTS

control_now
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	LEA	connowstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for config menu, and send
	MOVE.B	#'9',D1			* maximum number to accept
	BSR	num_wait		* get key character
	MOVE.L	D0,D2
	MOVE.W	#$8000,D1		* provide a small delay so that the
control_now_char_delay			* character can be seen on terminal
	DBF	D1,control_now_char_delay
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear screen
	SUB.B	#'1',D2			* convert ASCII number into binary
	CMP.B	#8,D2			* if 9 pressed, go to main menu
	BEQ	main_menu
	CMP.B	#7,D2			* if 8 pressed, get number of other
	BEQ	control_now_other_device * device and control it
	CMP.B	#6,D2			* if 7 pressed, get number of heater
	BEQ	control_now_heater_device * and control it
	BRA	not_implemented		* if other number, say isn't
					* implemented

control_now_other_device
	BSR	other_device
	BRA	control_device

control_now_heater_device
	BSR	heater_device
	BRA	control_device


not_implemented
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	LEA	notimpstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for message, and send
not_implemented_wait
	BSR     serial_getc
	BEQ	not_implemented_wait    * wait for key to be pressed
	BRA	main_menu

other_device
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	CLR	D3                      * minimum number
	MOVE.W	#$99,D4			* maximum number
	LEA	od1,A2			* get prompt
	BSR	get_bcd_number		* get the number - should be 0-255,
					* but is simpler if get_bcd is used
					* therefore only 0-99 available
	BSR	hex_to_dec		* convert to hex number in D0
	RTS				* return

heater_device
	MOVE.W	heater_dev_no,D0	* read heater device number
	RTS

control_device
	MOVE.L  D0,D6			* keep device number safe
	MOVEQ	#$67,D7			* code for control now
	BRA	set_event_get_setting	* find out what to set to

control_device_now			* D6=device number, D0=setting
	MOVE.L	D6,D1			* put number in D1
	EXG.L	D0,D1			* swap number and setting
	BSR	send_PIC_command	* send command packet
	BRA	main_menu		* then go to main menu

set_event
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for banner, and send
	LEA	setevstrtbl,A1		* point to table of string pointers
	BSR	send_stringtable	* for config menu, and send
	MOVE.B	#'9',D1			* maximum number to accept
	BSR	num_wait		* get key character
	MOVE.L	D0,D2
	MOVE.W	#$8000,D1		* provide a small delay so that the
set_event_char_delay			* character can be seen on terminal
	DBF	D1,set_event_char_delay
	MOVEQ	#FF,D0
	BSR	serial_putc		* clear screen
	SUB.B	#'1',D2			* convert ASCII number into binary
	CMP.B	#8,D2			* if 9 pressed, go to main menu
	BEQ	main_menu
	CMP.B	#7,D2			* if 8 pressed, get number of other
	BEQ	set_event_other_device	* device and control it
	CMP.B	#6,D2			* if 7 pressed, get number of heater
	BEQ	set_event_heater_device	* and control it
	BRA	not_implemented		* if other number, say isn't
					* implemented

set_event_other_device
	BSR	other_device
	BRA	set_event_get_details

set_event_heater_device
	BSR	heater_device
	BRA	set_event_get_details

set_event_get_details

	MOVE.L	D0,D2
	MOVEQ	#RTC_DEVICE_ALRM,D1	* device number byte in CMOS
	MOVE.W	#Hardware_RTC_Write,D0	* write it
	TRAP	#15

	MOVEQ	#FF,D0
	BSR	serial_putc		* clear the terminal screen
	LEA	banstrtbl,A1		* point to table of string pointers
        BSR	send_stringtable	* print banner
	LEA	setdetstrtbl,A1
        BSR	send_stringtable	* print title

	LEA	setdettbl,A3		* table pointing to prompt and regs
set_event_get_details_loop

	MOVEQ	#CR,D0
	BSR	serial_putc
	MOVEQ	#LF,D0
	BSR	serial_putc
	MOVEA.L	(A3)+,A2		* get prompt ptr
	MOVE.W	(A3)+,D3		* get min value
	MOVE.W	(A3)+,D4		* get max value
	BSR	get_bcd_number
	MOVE.W	(A3)+,D1		* RTC register
	MOVE.L	D0,D2			* value to store
	MOVE.W	#Hardware_RTC_Write,D0	* call OS routine
	TRAP	#15
	MOVE.L	(A3),D0			* get next address
	TST	D0			* is it zero?
	BNE	set_event_get_details_loop	* if not, get next value

set_event_get_setting
	LEA	newline,A0
	BSR	send_string		* print newline
	LEA	devctrlstrtbl,A1
	BSR	send_stringtable	* print menu
	MOVE.B	#'3',D1			* maximum number to accept
	BSR	num_wait		* get key character
	MOVE.L	D0,D2
	MOVE.W	#$8000,D1		* provide a small delay so that the
set_event_get_details_char_delay	* character can be seen on terminal
	DBF	D1,set_event_get_details_char_delay
	SUB.B	#'1',D2			* convert ASCII number into binary
	CMP.B	#0,D2			* if 1 pressed, turn off
	BEQ	set_event_gd_off
	CMP.B	#1,D2			* if 8 pressed, turn on
	BEQ	set_event_gd_on
					* here 2 was pressed, so get number
	LEA	newline,A1
	MOVEA.L	A1,A0
	BSR	send_string

	LEA	value,A2
	MOVEQ	#0,D3
	MOVE.W	#$99,D4
	BSR	get_bcd_number
	CMP	#$67,D7			* control immediately?
	BEQ	control_device_now
set_event_store_data
	MOVEQ	#RTC_DATA_VALUE,D1
	MOVE.L	D0,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15

set_event_enable_irq
	MOVEQ	#RTC_STATUS_B,D1	* RTC register B
	MOVE.W	#Hardware_RTC_Read,D0	* read it
	TRAP	#15
	BSET	#5,D0			* set alarm interrupt enable
	MOVE.L	D0,D2
	MOVEQ	#RTC_STATUS_B,D1	* RTC register B
	MOVE.W	#Hardware_RTC_Write,D0	* write it
	TRAP	#15
	BRA	main_menu

set_event_gd_off
	MOVEQ	#0,D0
	BRA	set_event_store_data

set_event_gd_on
	MOVE.W	#$99,D0
	BRA	set_event_store_data

send_PIC_command			* D0 = device number, D1 = value
	BSR	send_PIC_byte		* send D0
	MOVE.L	D1,D0
	BSR	send_PIC_byte		* send D1
	MOVE.W	#$FF,D0
	BSR	send_PIC_byte		* send end of packet code
	RTS

send_PIC_byte
	MOVE.B	D0,VIA_DRA		* write value to PIC through VIA
	MOVE.W	#$FFF,D7		* delay value
send_PIC_byte_loop			* delay waiting for IRQ
	DBF	D7,send_PIC_byte_loop
	RTS


* Initialisation section for main program

main_init
	MOVE.W	#$0,SR			* clear status register - go into
					* user mode

	MOVEA.L	#$43F00,A7		* set user stack pointer to
					* sensible RAM address
	BRA	 start_init		* return - can't use BSR/RTS because
					* stacks have been swapped

pia_init
	MOVE.B	#$31,PIA_CRA		* access DDRA, enable CA1
					* and system interrupts
	MOVE.B	#$FF,PIA_DDRA		* make PIA Port A outputs
	MOVE.B	#$00,PIA_CRB		* set PB0-3 outputs,
	MOVE.B  #$0F,PIA_DDRB		*     PB4-7 inputs
	MOVE.B	#$04,PIA_CRB		* select Peripheral Reg B
	MOVE.B	#$F9,PIA_PRB		* other bits high, R-/W high,
					* DS, AS low, /CS high
	RTS				* return

serial_init
	MOVE.B	DIN,D0			* read input port
	AND.B	#$7F,D0			* mask off other bits
	MOVE.B	D0,D1			* take copy of D0
	AND.B	#$7,D0			* mask only bps selector bits
	MOVE.B	D0,SER_BPS		* set serial data rate

	MOVE.B	#3,ACIA_CR		* Initialise 6850

	MOVEQ	#2,D2			* select divide by 64

	BTST	#3,D1			* is bit 3 of DIP switch on?
	BNE.S	serial_init_setdiv16	* if so select div by 16
serial_init_cont1
	AND.B	#$70,D1			* get word select bits
	LSR.B	#2,D1			* shift into correct position for
					* ACIA control register
	OR.B	D1,D2			* OR with D2 -> D2
	*OR.B	#$00,D2			* CR7-5 = 101, RTS on, enable
					* receive and transmit IRQs
	*MOVE.B	D2,DOUT_L		* DEBUG CODE
	MOVE.B	D2,ACIA_CR		* set control register
	MOVE.B	D2,SERIAL_CONFIG
	CLR.W	SERIAL_RX_INPTR		* Reset buffer pointers
	CLR.W	SERIAL_RX_OUTPTR
	CLR.W	SERIAL_TX_INPTR
	CLR.W	SERIAL_TX_OUTPTR
	RTS				* return from subroutine

serial_init_setdiv16
	MOVEQ	#1,D2			* select divide by 16
	*MOVE.B  #17,DOUT_H		* DEBUG
	BRA	serial_init_cont1

rtc_init
	MOVEQ	#RTC_STATUS_D,D1	* RTC status register D
	MOVE.W	#Hardware_RTC_Read,D0	* read it
	TRAP	#15
	MOVE.L	D0,D3			* Preserve result

	MOVEQ	#RTC_STATUS_A,D1	* RTC status register A
	MOVEQ	#$00,D2			* clear it
	MOVE.W	#Hardware_RTC_Write,D0	* call OS routine
	TRAP	#15
	MOVEQ	#RTC_STATUS_C,D1	* RTC status register C
	MOVE.W	#Hardware_RTC_Read,D0	* reset interrupt flags
	TRAP	#15			* call OS routine
	MOVEQ	#RTC_STATUS_B,D1	* RTC status register B
	MOVEQ	#$10,D2			* set Update Interrupt Enabled
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15

	BTST	#7,D3			* is bit 7 of reg D (VRT) set?
	BNE	rtc_init_end		* if so, don't bother to reset time

	MOVEQ	#RTC_SECS,D1		* on startup, set time and date
	MOVEQ	#$00,D2			* to 00:00:00 on 1st January 1997
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_MINS,D1
	MOVEQ	#$00,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_HRS,D1
	MOVEQ	#$00,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_DAY,D1
	MOVEQ	#$1,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_DATE,D1
	MOVEQ	#$1,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_MONTH,D1
	MOVEQ	#$1,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15
	MOVEQ	#RTC_YEAR,D1
	MOVE.b	#$97,D2
	MOVE.W	#Hardware_RTC_Write,D0
	TRAP	#15

rtc_init_end
        RTS

via_init				* VIA initialisation
	CLR.B	VIA_DDRA		* Make port A an input
	CLR.B	VIA_DDRB		* Make port B an input
	MOVE.B	#%00001000,VIA_PCR	* Select port A handshake mode
	MOVE.B	#%10010000,VIA_IER	* Enable CB1 interrupt
	RTS				* Return

program_init
	MOVE.L	#TF_BLK,TF_START	* initialise timeframe list
	MOVE.L	#TF_BLK,TF_END
	RTS

* Miscellaneous general purpose subroutines

serial_putc
	BTST	#1,ACIA_SR		* is port 1 ready for a character?
	BEQ	serial_putc		* if not, wait for it
	MOVE.B	D0,ACIA_TR		* out it goes.
	RTS

serial_getc
	BTST	#0,ACIA_SR		* is character ready?
	BEQ.S	serial_getc_ret		* if not, return Zero status
	MOVE.B	ACIA_RR,D0		* else get the character
	AND.B	#$7F,D0			* zero out the high bit
serial_getc_ret
	RTS


send_string				* Send serial string pointed to by A0
	MOVE.B	(A0)+,D0		* get next byte
	TST.B	D0
	BEQ	send_string_return
	BSR	serial_putc		* send it
	BRA	send_string		* if not do next one
send_string_return
	RTS

send_bcd_number
	MOVE.L	D1,-(A7)
	MOVE.L	D0,D1
	LSR	#4,D0
	AND	#$0F,D0
	ADD.B	#'0',D0
	BSR	serial_putc
	MOVE.L	D1,D0
	AND	#$0F,D0
	ADD.B	#'0',D0
	BSR	serial_putc
	MOVE.L	(A7)+,D1
	RTS

send_stringtable			* send an whole string table through
					* the serial port
					* on entry A1 points to table
	MOVEM.L	D1/A0,-(A7)
	CLR	D1
send_stringtable_loop
	MOVEA.L	(A1,D1),A0		* get address from table
	CMPA	#0,A0
	BEQ	send_stringtable_end	* if zero, end procedure
	BSR	send_string		* if not, send string
	ADDQ	#4,D1
	BRA	send_stringtable_loop
send_stringtable_end
	MOVEM.L	(A7)+,D1/A0		* restore registers
	RTS				* return

get_line				* read a line, terminated by control
					* character from serial port
	LEA	SERIAL_WORKSPACE,A0	* buffer to store string
get_line_loop
	BSR	serial_getc		* get character
	BEQ	get_line_loop
	CMP.B	#127,D0			* is it delete/backspace
	BEQ	get_line_del		* if so delete character
	CMP.B	#8,D0
	BEQ	get_line_del
	BSR	serial_putc		* otherwise echo it to the terminal
	MOVE.B	D0,(A0)+
	CMP.B	#31,D0			* is it control char?
	BGT	get_line_loop		* if not, get next
	MOVE.B	#0,-1(A0)		* replace control char with zero
	LEA	SERIAL_WORKSPACE,A0	* return start of string
	RTS

get_line_del
	SUBQ	#1,A0			* delete previous char
	CMPA.L	#SERIAL_WORKSPACE,A0	* is it before workspace
	BLT.S	get_line		* if so, start again
	MOVEQ	#8,D0			* backspace
	BSR	serial_putc
	MOVEQ	#' ',D0			* overwrite last char
	BSR	serial_putc
	MOVEQ	#8,D0			* backspace
	BSR	serial_putc		* delete character on terminal
	BRA.S	get_line_loop		* and delete char


rtc_sendtime
	LEA     tsec,A0
	BSR	send_string
	MOVE.B	RTC_SC_SECS,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     tmin,A0
	BSR	send_string
	MOVE.B	RTC_SC_MINS,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     thrs,A0
	BSR	send_string
	MOVE.B	RTC_SC_HRS,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     tday,A0
	BSR	send_string
	MOVE.B	RTC_SC_DAY,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     tdate,A0
	BSR	send_string
	MOVE.B	RTC_SC_DATE,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     tmonth,A0
	BSR	send_string
	MOVE.B	RTC_SC_MONTH,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	LEA     tyear,A0
	BSR	send_string
	MOVE.B	RTC_SC_YEAR,D0
	BSR	send_bcd_number
	LEA	newline,A0
	BSR	send_string
	RTS
rtc_showtime
        MOVEQ   #FF,D0
	BSR	serial_putc
rtc_showloop
	BSR	rtc_sendtime
	MOVE.W	#$1000,D0
rl
	DBF	D0,rl
	lea	up,A0
	BSR	send_string
	BSR	serial_getc
	BNE	main_menu
	BRA	rtc_showloop
	DC.L 0

hex_to_dec
	MOVE.W	D0,D1		* D0 = BCD value
				* D1 = decimal result
	AND.W	#$F,D1		* mask off high bits
	AND.W	#$F0,D0		* mask off low bits of D0
	LSR.W	#4,D0		* make into nybble
	MOVE.L	D0,D2		* D2 is temp reg
	LSL.W	#3,D2		* D0 * 8
	ADD.W	D0,D2		* D0 * 8 + D0
	ADD.W	D0,D2		* D0 * 8 + D0 + D0 = 10 D0
	ADD.W	D1,D2		* add to low nybble and ret in D0
	MOVE.W	D2,D0
	RTS

* Data and text messages

heater_dev_no
	DC.W	$0007			* PIC id number for heater
	DCB.W	0,0
tb
	DC.B	'Home Control System',LF,CR,LF,CR,0
	DCB.W	0,0

ver
	DC.B	'Version 1.10 (9 March 1997)',LF,CR,LF,CR,LF,CR,0
	DCB.W	0,0

* Main menu options
mm
mm1	DC.B	'1)	Configure system',LF,CR,LF,CR,0
mm2	DC.B	'2)	Control appliance now',LF,CR,LF,CR,0
mm3	DC.B	'3)	Set timed event',LF,CR,LF,CR,0
mm4	DC.B	'4)	List current timed events',LF,CR,LF,CR,0
mm5	DC.B	'5)	Clear timed event',LF,CR,LF,CR,0
mm6	DC.B	'6)	View current conditions',LF,CR,LF,CR,0
mm7	DC.B	'7)	Goodbye',LF,CR,LF,CR,0
gap	DC.B	LF,CR,0
plent	DC.B	'Please enter a number : ',0
	DC.W	0

banstrtbl
	DC.L	tb
	DC.L	ver
	DC.L	0
mmstrtbl
	DC.L	mm1
	DC.L	mm2
	DC.L	mm3
	DC.L	mm4
	DC.L	mm5
	DC.L	mm6
	DC.L	mm7
	DC.L	gap
	DC.L	plent
	DC.L	0

cs
cs1	DC.B	'1)	Set system clock',LF,CR,LF,CR,0
cs2	DC.B	'2)	Return to main menu',LF,CR,LF,CR,0
	DC.L	0

configstrtbl
	DC.L	cs1
	DC.L	cs2
	DC.L	gap
	DC.L	plent
	DC.L	0

sc
sc1	DC.B	'Set system clock',LF,CR,0
sc2	DC.B	'----------------',LF,CR,0
	DC.W	0

clksetstrtbl
	DC.L	sc1
	DC.L	sc2
	DC.L	0

modem_cmdmode
	DC.B	'+++',0
modem_hangup
	DC.B	'ATH',CR,0
	dc.l	0

tsec	DC.B	'Seconds: ',0
tmin	DC.B	'Minutes: ',0
thrs	DC.B	'Hours: ',0
tday	DC.B	'Day of week: ',0
tdate	DC.B	'Day of month: ',0
tmonth	DC.B	'Month: ',0
tyear	DC.B	'Year: 19',0
	DCB.W	0,0

rtcsettbl
	DC.L	tyear
	DC.W	$00,$99,RTC_YEAR
	DC.L	tmonth
	DC.W	$01,$12,RTC_MONTH
	DC.L	tdate
	DC.W	$01,$31,RTC_DATE
	DC.L	thrs
	DC.W	$00,$23,RTC_HRS
	DC.L	tmin
	DC.W	$00,$59,RTC_MINS
	DC.L	tsec
	DC.W	$00,$59,RTC_SECS
	DC.L	0
	DC.W	0,0

cn	DC.B	'Control device now',LF,CR,0
cns	DC.B	'------------------',LF,CR,LF,CR,0
ct1	DC.B	'1)	Central heating',LF,CR,0
ct2	DC.B	'2)	Outside lights',LF,CR,0
ct3	DC.B	'3)	Ground floor lights',LF,CR,0
ct4	DC.B	'4)	Ground floor appliances',LF,CR,0
ct5	DC.B	'5)	First floor lights',LF,CR,0
ct6	DC.B	'6)	First floor appliances',LF,CR,0
ct7	DC.B	'7)	Immersion heater',LF,CR,0
ct8	DC.B	'8)	Other electrical devices',LF,CR,LF,CR,0
ct9	DC.B	'9)	Return to main menu',LF,CR,LF,CR,0
	DCB.W	0,0

connowstrtbl
	DC.L	cn
	DC.L	cns
	DC.L	ct1
	DC.L	ct2
	DC.L	ct3
	DC.L	ct4
	DC.L	ct5
	DC.L	ct6
	DC.L	ct7
	DC.L	ct8
	DC.L	ct9
	DC.L	gap
	DC.L	plent
	DC.L	0

se	DC.B	'Set timed event',LF,CR,0
ses	DC.B	'---------------',LF,CR,LF,CR,0

        DCB.W   0,0
setevstrtbl
	DC.L	se
	DC.L	ses
	DC.L	ct1
	DC.L	ct2
	DC.L	ct3
	DC.L	ct4
	DC.L	ct5
	DC.L	ct6
	DC.L	ct7
	DC.L	ct8
	DC.L	ct9
	DC.L	gap
	DC.L	plent
	DC.L	0


        DCB.W   0,0
ni1	DC.B	'This function is not implemented in this version',LF,CR,0
ni2	DC.B	'of the control software. Currently, only the',LF,CR,0
ni3	DC.B	'immersion heater can be controlled.',LF,CR,LF,CR,0
ni4	DC.B	'If this system were being used commercially,',LF,CR,0
ni5	DC.B	'these functions would be present and the software',LF,CR,0
ni6	DC.B	'would be considerably refined.',LF,CR,LF,CR,0
nip	DC.B	'Press any key to continue.',LF,CR,0

        DCB.W   0,0
notimpstrtbl
	DC.L	ni1
	DC.L	ni2
	DC.L	ni3
	DC.L	ni4
	DC.L	ni5
	DC.L	ni6
	DC.L	nip
	DC.L	0

	DCB.W	0,0
od1	DC.B	'Enter number of device to control: ',0

setdetstrtbl
	DC.L	se
	DC.L	ses
	DC.L	0

setdettbl
	DC.L	thrs
	DC.W	$00,$23,RTC_HRS_ALRM
	DC.L	tmin
	DC.W	$00,$59,RTC_MINS_ALRM
	DC.L	tsec
	DC.W	$00,$59,RTC_SECS_ALRM
	*DC.L	value
	*DC.W	$00,$99,$0F
	DC.L	0
	DC.W	0,0

dc
dc1	DC.B	'Device control:',LF,CR,LF,CR,0
dc2	DC.B	'1) Device off',LF,CR,0
dc3	DC.B	'2) Device on',LF,CR,0
dc4	DC.B	'3) Proportional control',LF,CR,0
	DCB.W	0,0

devctrlstrtbl
	DC.L	dc1
	DC.L	dc2
	DC.L	dc3
	DC.L	dc4
	DC.L	gap
	DC.L	plent
	DC.L	0

proptbl
	DC.L	value
	DC.W	$00,$99,$0F
	DC.L	0,0

value	DC.B	'Value to send to device: ',0
	DCB.W	0,0

newline	DC.B	CR,LF,0

up	DC.B 	27,'[1A',27,'[1A',27,'[1A',27,'[1A',27,'[1A'
	DC.B	27,'[1A',27,'[1A',0

wipestr	DCB.B	79,' '
	DCB.B	79,8
	DC.B	0

cursorup
	DC.B	27,'[1A',LF,0
	END start
